@@ -4,7 +4,7 @@ use crate::flags::{Display, Flags, HyperlinkOption, Layout};
 use crate::git_theme::GitTheme;
 use crate::icon::Icons;
 use crate::meta::name::DisplayOption;
-use crate::meta::{FileType, Meta};
+use crate::meta::{FileType, Meta, OwnerCache};
 use std::collections::HashMap;
 use term_grid::{Cell, Direction, Filling, Grid, GridOptions};
 use terminal_size::terminal_size;
@@ -23,10 +23,12 @@ pub fn grid(
     git_theme: &GitTheme,
 ) -> String {
     let term_width = terminal_size().map(|(w, _)| w.0 as usize);
+    let owner_cache = OwnerCache::default();
 
     inner_display_grid(
         &DisplayOption::None,
         metas,
+        &owner_cache,
         flags,
         colors,
         icons,
@@ -57,8 +59,11 @@ pub fn tree(
         }
     }
 
+    let owner_cache = OwnerCache::default();
+
     for cell in inner_display_tree(
         metas,
+        &owner_cache,
         flags,
         colors,
         icons,
@@ -77,6 +82,7 @@ pub fn tree(
 fn inner_display_grid(
     display_option: &DisplayOption,
     metas: &[Meta],
+    owner_cache: &OwnerCache,
     flags: &Flags,
     colors: &Colors,
     icons: &Icons,
@@ -109,14 +115,15 @@ fn inner_display_grid(
         // Maybe skip showing the directory meta now; show its contents later.
         if skip_dirs
             && (matches!(meta.file_type, FileType::Directory { .. })
-                || (matches!(meta.file_type, FileType::SymLink { is_dir: true })
-                    && flags.layout != Layout::OneLine))
+                || (matches!(meta.file_type, FileType::SymLink { is_dir: true }))
+                    && flags.blocks.0.len() == 1)
         {
             continue;
         }
 
         let blocks = get_output(
             meta,
+            owner_cache,
             colors,
             icons,
             git_theme,
@@ -160,7 +167,7 @@ fn inner_display_grid(
         output += &grid.fit_into_columns(flags.blocks.0.len()).to_string();
     }
 
-    let should_display_folder_path = should_display_folder_path(depth, metas, flags);
+    let should_display_folder_path = should_display_folder_path(depth, metas);
 
     // print the folder content
     for meta in metas {
@@ -176,6 +183,7 @@ fn inner_display_grid(
             output += &inner_display_grid(
                 &display_option,
                 content,
+                owner_cache,
                 flags,
                 colors,
                 icons,
@@ -223,6 +231,7 @@ fn add_header(flags: &Flags, cells: &[Cell], grid: &mut Grid) {
 #[allow(clippy::too_many_arguments)]
 fn inner_display_tree(
     metas: &[Meta],
+    owner_cache: &OwnerCache,
     flags: &Flags,
     colors: &Colors,
     icons: &Icons,
@@ -248,6 +257,7 @@ fn inner_display_tree(
 
         for block in get_output(
             meta,
+            owner_cache,
             colors,
             icons,
             git_theme,
@@ -276,6 +286,7 @@ fn inner_display_tree(
 
             cells.extend(inner_display_tree(
                 content,
+                owner_cache,
                 flags,
                 colors,
                 icons,
@@ -290,7 +301,7 @@ fn inner_display_tree(
     cells
 }
 
-fn should_display_folder_path(depth: usize, metas: &[Meta], flags: &Flags) -> bool {
+fn should_display_folder_path(depth: usize, metas: &[Meta]) -> bool {
     if depth > 0 {
         true
     } else {
@@ -298,8 +309,7 @@ fn should_display_folder_path(depth: usize, metas: &[Meta], flags: &Flags) -> bo
             .iter()
             .filter(|x| {
                 matches!(x.file_type, FileType::Directory { .. })
-                    || (matches!(x.file_type, FileType::SymLink { is_dir: true })
-                        && flags.layout != Layout::OneLine)
+                    || (matches!(x.file_type, FileType::SymLink { is_dir: true }))
             })
             .count();
 
@@ -314,6 +324,7 @@ fn display_folder_path(meta: &Meta) -> String {
 #[allow(clippy::too_many_arguments)]
 fn get_output(
     meta: &Meta,
+    owner_cache: &OwnerCache,
     colors: &Colors,
     icons: &Icons,
     git_theme: &GitTheme,
@@ -357,11 +368,11 @@ fn get_output(
                 ]);
             }
             Block::User => block_vec.push(match &meta.owner {
-                Some(owner) => owner.render_user(colors, flags),
+                Some(owner) => owner.render_user(colors, owner_cache, flags),
                 None => colorize_missing("?"),
             }),
             Block::Group => block_vec.push(match &meta.owner {
-                Some(owner) => owner.render_group(colors, flags),
+                Some(owner) => owner.render_group(colors, owner_cache, flags),
                 None => colorize_missing("?"),
             }),
             Block::Context => block_vec.push(match &meta.access_control {
@@ -913,23 +924,20 @@ mod tests {
         const NO: bool = false;
 
         assert_eq!(
-            should_display_folder_path(0, &[file.clone()], &Flags::default()),
+            should_display_folder_path(0, &[file.clone()]),
             YES // doesn't matter since there is no folder
         );
+        assert_eq!(should_display_folder_path(0, &[dir.clone()]), NO);
         assert_eq!(
-            should_display_folder_path(0, &[dir.clone()], &Flags::default()),
-            NO
-        );
-        assert_eq!(
-            should_display_folder_path(0, &[file.clone(), dir.clone()], &Flags::default()),
+            should_display_folder_path(0, &[file.clone(), dir.clone()]),
             YES
         );
         assert_eq!(
-            should_display_folder_path(0, &[dir.clone(), dir.clone()], &Flags::default()),
+            should_display_folder_path(0, &[dir.clone(), dir.clone()]),
             YES
         );
         assert_eq!(
-            should_display_folder_path(0, &[file.clone(), file.clone()], &Flags::default()),
+            should_display_folder_path(0, &[file.clone(), file.clone()]),
             YES // doesn't matter since there is no folder
         );
 
@@ -954,43 +962,18 @@ mod tests {
         std::os::unix::fs::symlink("dir", &link_path).unwrap();
         let link = Meta::from_path(&link_path, false, PermissionFlag::Rwx).unwrap();
 
-        let grid_flags = Flags {
-            layout: Layout::Grid,
-            ..Flags::default()
-        };
-
-        let oneline_flags = Flags {
-            layout: Layout::OneLine,
-            ..Flags::default()
-        };
-
         const YES: bool = true;
         const NO: bool = false;
 
+        assert_eq!(should_display_folder_path(0, &[link.clone()]), NO);
+
         assert_eq!(
-            should_display_folder_path(0, &[link.clone()], &grid_flags),
-            NO
-        );
-        assert_eq!(
-            should_display_folder_path(0, &[link.clone()], &oneline_flags),
-            YES // doesn't matter since this link will be expanded as a directory
+            should_display_folder_path(0, &[file.clone(), link.clone()]),
+            YES
         );
 
         assert_eq!(
-            should_display_folder_path(0, &[file.clone(), link.clone()], &grid_flags),
-            YES
-        );
-        assert_eq!(
-            should_display_folder_path(0, &[file.clone(), link.clone()], &oneline_flags),
-            YES // doesn't matter since this link will be expanded as a directory
-        );
-
-        assert_eq!(
-            should_display_folder_path(0, &[dir.clone(), link.clone()], &grid_flags),
-            YES
-        );
-        assert_eq!(
-            should_display_folder_path(0, &[dir.clone(), link.clone()], &oneline_flags),
+            should_display_folder_path(0, &[dir.clone(), link.clone()]),
             YES
         );
 
